/**
	\file openobex/obex.h
	OpenOBEX library - Free implementation of the Object Exchange protocol.

	Copyright (C) 1999-2000  Dag Brattli <dagb@cs.uit.no>
	Copyright (C) 1999-2000  Pontus Fuchs <pontus.fuchs@tactel.se>
	Copyright (C) 2001-2002  Jean Tourrilhes <jt@hpl.hp.com>
	Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
	Copyright (C) 2002-2008  Christian W. Zuckschwerdt <zany@triq.net>
	Copyright (C) 2002  Dmitry Kasatkin <dmitry.kasatkin@nokia.com>
	Copyright (C) 2005  Herton Ronaldo Krzesinski <herton@conectiva.com.br>
	Copyright (C) 2005-2008  Alex Kanavin <ak@sensi.org>
	Copyright (C) 2006  Johan Hedberg <johan.hedberg@nokia.com>
	Copyright (C) 2007-2008  Hendrik Sattler <post@hendrik-sattler.de>

	OpenOBEX is free software; you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as
	published by the Free Software Foundation; either version 2.1 of
	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with OpenOBEX. If not, see <http://www.gnu.org/>.
 */

#ifndef OPENOBEX_OBEX_H
#define OPENOBEX_OBEX_H

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _WIN32
#include <winsock2.h>
#ifdef OPENOBEX_DLL
#define OPENOBEX_SYMBOL(retval) __declspec(dllimport) retval WINAPI
#else
#define OPENOBEX_SYMBOL(retval) retval WINAPI
#endif

#else /* _WIN32 */
#include <sys/socket.h>
#endif /* _WIN32 */

#include <inttypes.h>

#ifndef OPENOBEX_SYMBOL
#define OPENOBEX_SYMBOL(retval) retval
#endif

/* Hum... This would need to be autogenerated from configure,
 * I hate hardcoding version numbers. Jean II */
#define OPENOBEX_VERSION(a, b, c) (((a) << 16) + ((b) << 8) + (c))

// These are also defined in obex_main.h
// They are defined here otherwise any app including it
// would require obex_main.h which includes a lot of other 
// unnecessary stuff.
typedef void* obex_t;
typedef void* obex_object_t;
typedef void (*obex_event_t)(obex_t *handle, obex_object_t *obj, int mode, int event, int obex_cmd, int obex_rsp);
// This is to workaround compilation without Bluetooth support. - Jean II

#include <openobex/obex_const.h>

/*
 *  OBEX API
 */
OPENOBEX_SYMBOL(obex_t *) OBEX_Init(int transport, obex_event_t eventcb, unsigned int flags);
OPENOBEX_SYMBOL(void)     OBEX_Cleanup(obex_t *self);
OPENOBEX_SYMBOL(void)     OBEX_SetUserData(obex_t *self, void * data);
OPENOBEX_SYMBOL(void *)   OBEX_GetUserData(obex_t *self);
OPENOBEX_SYMBOL(void)     OBEX_SetUserCallBack(obex_t *self, obex_event_t eventcb, void * data);
OPENOBEX_SYMBOL(int)      OBEX_SetTransportMTU(obex_t *self, uint16_t mtu_rx, uint16_t mtu_tx_max);
OPENOBEX_SYMBOL(int)      OBEX_GetFD(obex_t *self);

OPENOBEX_SYMBOL(int)    OBEX_RegisterCTransport(obex_t *self, obex_ctrans_t *ctrans);
OPENOBEX_SYMBOL(void)   OBEX_SetCustomData(obex_t *self, void * data);
OPENOBEX_SYMBOL(void *) OBEX_GetCustomData(obex_t *self);

OPENOBEX_SYMBOL(int) OBEX_TransportConnect(obex_t *self, struct sockaddr *saddr, int addlen);
OPENOBEX_SYMBOL(int) OBEX_TransportDisconnect(obex_t *self);
OPENOBEX_SYMBOL(int) OBEX_CustomDataFeed(obex_t *self, uint8_t *inputbuf, int actual);
OPENOBEX_SYMBOL(int) OBEX_HandleInput(obex_t *self, int timeout);

OPENOBEX_SYMBOL(int)      OBEX_ServerRegister(obex_t *self, struct sockaddr *saddr, int addrlen);
OPENOBEX_SYMBOL(obex_t *) OBEX_ServerAccept(obex_t *server, obex_event_t eventcb, void * data);

OPENOBEX_SYMBOL(int) OBEX_Request(obex_t *self, obex_object_t *object);
OPENOBEX_SYMBOL(int) OBEX_CancelRequest(obex_t *self, int nice);
OPENOBEX_SYMBOL(int) OBEX_SuspendRequest(obex_t *self, obex_object_t *object);
OPENOBEX_SYMBOL(int) OBEX_ResumeRequest(obex_t *self);

OPENOBEX_SYMBOL(obex_object_t *) OBEX_ObjectNew(obex_t *self, uint8_t cmd);
OPENOBEX_SYMBOL(int)             OBEX_ObjectDelete(obex_t *self, obex_object_t *object);

OPENOBEX_SYMBOL(int) OBEX_ObjectAddHeader(obex_t *self, obex_object_t *object,
				    uint8_t hi, obex_headerdata_t hv, uint32_t hv_size,
				    unsigned int flags);
OPENOBEX_SYMBOL(int) OBEX_ObjectGetNextHeader(obex_t *self, obex_object_t *object,
					uint8_t *hi, obex_headerdata_t *hv, uint32_t *hv_size);
OPENOBEX_SYMBOL(int) OBEX_ObjectReParseHeaders(obex_t *self, obex_object_t *object);
OPENOBEX_SYMBOL(int) OBEX_ObjectSetRsp(obex_object_t *object, uint8_t rsp, uint8_t lastrsp);

OPENOBEX_SYMBOL(int) OBEX_ObjectGetNonHdrData(obex_object_t *object, uint8_t **buffer);
OPENOBEX_SYMBOL(int) OBEX_ObjectSetNonHdrData(obex_object_t *object, const uint8_t *buffer, unsigned int len);
OPENOBEX_SYMBOL(int) OBEX_ObjectSetHdrOffset(obex_object_t *object, unsigned int offset);
OPENOBEX_SYMBOL(int) OBEX_ObjectReadStream(obex_t *self, obex_object_t *object, const uint8_t **buf);
OPENOBEX_SYMBOL(int) OBEX_ObjectGetCommand(obex_t *self, obex_object_t *object);

OPENOBEX_SYMBOL(int) OBEX_UnicodeToChar(uint8_t *c, const uint8_t *uc, int size);
OPENOBEX_SYMBOL(int) OBEX_CharToUnicode(uint8_t *uc, const uint8_t *c, int size);

OPENOBEX_SYMBOL(char *) OBEX_ResponseToString(int rsp);

/*
 * This function is deprecated and will be removed in OpenOBEX 1.1.0
 *
 * Please use the OBEX_ResponseToString instead.
 *
 */
OPENOBEX_SYMBOL(char *) OBEX_GetResponseMessage(obex_t *self, int rsp);

/*
 * TcpOBEX API (IPv4/IPv6)
 */
OPENOBEX_SYMBOL(int) TcpOBEX_ServerRegister(obex_t *self, struct sockaddr *addr, int addrlen);
OPENOBEX_SYMBOL(int) TcpOBEX_TransportConnect(obex_t *self, struct sockaddr *addr, int addrlen);

/*
 * InOBEX API (deprecated, maps to TcpOBEX_*)
 */
OPENOBEX_SYMBOL(int) InOBEX_ServerRegister(obex_t *self);
OPENOBEX_SYMBOL(int) InOBEX_TransportConnect(obex_t *self, struct sockaddr *saddr, int addrlen);

/*
 * IrOBEX API 
 */
OPENOBEX_SYMBOL(int) IrOBEX_ServerRegister(obex_t *self, const char *service);
OPENOBEX_SYMBOL(int) IrOBEX_TransportConnect(obex_t *self, const char *service);

/*
 * Bluetooth OBEX API
 */
#ifdef SOL_RFCOMM
#ifdef _WIN32
#define bdaddr_t BTH_ADDR
#endif
OPENOBEX_SYMBOL(int) BtOBEX_ServerRegister(obex_t *self, bdaddr_t *src, uint8_t channel);
OPENOBEX_SYMBOL(int) BtOBEX_TransportConnect(obex_t *self, bdaddr_t *src, bdaddr_t *dst, uint8_t channel);
#endif

/*
 * OBEX File API
 */
OPENOBEX_SYMBOL(int) FdOBEX_TransportSetup(obex_t *self, int rfd, int wfd, int mtu);

/*  
 * OBEX interface discovery API 
 */
OPENOBEX_SYMBOL(int)  OBEX_FindInterfaces(obex_t *self, obex_interface_t **intf);
OPENOBEX_SYMBOL(int)  OBEX_InterfaceConnect(obex_t *self, obex_interface_t *intf);
OPENOBEX_SYMBOL(void) OBEX_FreeInterfaces(obex_t *self);

#ifdef __cplusplus
}
#endif

#endif /* OPENOBEX_OBEX_H */
